@@ -31,9 +31,7 @@
 #include <vector>
 #include <memory>
 #include <unistd.h>
-
-#include <unicode/translit.h>
-#include <unicode/ucnv.h>
+#include <algorithm>
 
 #ifdef _WIN32
 #define SEPARATOR "\\"
@@ -42,9 +40,6 @@
 #endif
 
 
-extern bool uniqueNamespace;
-
-
 unsigned int getFileSize(const std::string& path)
 {
   struct stat filestatus;
@@ -64,6 +59,13 @@ bool fileExists(const std::string& path)
   return flag;
 }
 
+bool isDirectory(const std::string &path)
+{
+  struct stat filestatus;
+  stat(path.c_str(), &filestatus);
+  return (filestatus.st_mode & S_IFMT) == S_IFDIR;
+}
+
 /* base64 */
 static const std::string base64_chars
     = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -130,13 +132,17 @@ std::string decodeUrl(const std::string& originalUrl)
   std::string::size_type pos = 0;
   while ((pos = url.find('%', pos)) != std::string::npos
          && pos + 2 < url.length()) {
+    if (!isxdigit(url[pos+1]) || !isxdigit(url[pos+2])) {
+      ++pos;
+      continue;
+    }
     url.replace(pos, 3, 1, charFromHex(url.substr(pos + 1, 2)));
     ++pos;
   }
   return url;
 }
 
-std::string removeLastPathElement(const std::string& path,
+static std::string removeLastPathElement(const std::string& path,
                                   const bool removePreSeparator,
                                   const bool removePostSeparator)
 {
@@ -154,7 +160,7 @@ std::string removeLastPathElement(const std::string& path,
 }
 
 /* Split string in a token array */
-std::vector<std::string> split(const std::string& str,
+static std::vector<std::string> split(const std::string& str,
                                const std::string& delims = " *-")
 {
   std::string::size_type lastPos = str.find_first_not_of(delims, 0);
@@ -170,18 +176,13 @@ std::vector<std::string> split(const std::string& str,
   return tokens;
 }
 
-std::vector<std::string> split(const char* lhs, const char* rhs)
+static std::vector<std::string> split(const char* lhs, const char* rhs)
 {
   const std::string m1(lhs), m2(rhs);
   return split(m1, m2);
 }
 
-std::vector<std::string> split(const char* lhs, const std::string& rhs)
-{
-  return split(lhs, rhs.c_str());
-}
-
-std::vector<std::string> split(const std::string& lhs, const char* rhs)
+static std::vector<std::string> split(const std::string& lhs, const char* rhs)
 {
   return split(lhs.c_str(), rhs);
 }
@@ -216,6 +217,9 @@ std::string computeAbsolutePath(const std::string& path,
 std::string computeRelativePath(const std::string path,
                                 const std::string absolutePath)
 {
+  if (path.empty())
+    return "";
+
   std::vector<std::string> pathParts = split(path, "/");
   std::vector<std::string> absolutePathParts = split(absolutePath, "/");
 
@@ -256,6 +260,9 @@ void replaceStringInPlace(std::string& subject,
                           const std::string& search,
                           const std::string& replace)
 {
+  if (search.empty())
+    return;
+
   size_t pos = 0;
   while ((pos = subject.find(search, pos)) != std::string::npos) {
     subject.replace(pos, search.length(), replace);
@@ -272,7 +279,7 @@ void stripTitleInvalidChars(std::string& str)
   replaceStringInPlace(str, "\u202C", "");
 }
 
-std::string getNamespaceForMimeType(const std::string& mimeType)
+std::string getNamespaceForMimeType(const std::string& mimeType, bool uniqueNamespace)
 {
   if (uniqueNamespace || mimeType.find("text") == 0 || mimeType.empty()) {
     if (uniqueNamespace || mimeType.find("text/html") == 0
@@ -296,20 +303,6 @@ std::string getNamespaceForMimeType(const std::string& mimeType)
   }
 }
 
-
-std::string removeAccents(const std::string& text)
-{
-  ucnv_setDefaultName("UTF-8");
-  static UErrorCode status = U_ZERO_ERROR;
-  static std::unique_ptr<icu::Transliterator> removeAccentsTrans(icu::Transliterator::createInstance(
-      "Lower; NFD; [:M:] remove; NFC", UTRANS_FORWARD, status));
-  icu::UnicodeString ustring(text.c_str());
-  removeAccentsTrans->transliterate(ustring);
-  std::string unaccentedText;
-  ustring.toUTF8String(unaccentedText);
-  return unaccentedText;
-}
-
 void remove_all(const std::string& path)
 {
   DIR* dir;
@@ -332,3 +325,148 @@ void remove_all(const std::string& path)
     remove(path.c_str());
   }
 }
+
+std::vector<html_link> generic_getLinks(const std::string& page)
+{
+    const char* p = page.c_str();
+    const char* linkStart;
+    std::vector<html_link> links;
+    std::string attr;
+
+    while (*p) {
+        if (strncmp(p, " href", 5) == 0) {
+            attr = "href";
+            p += 5;
+        } else if (strncmp(p, " src", 4) == 0) {
+            attr = "src";
+            p += 4;
+        } else {
+            p += 1;
+            continue;
+        }
+
+        while (*p == ' ')
+            p += 1 ;
+        if (*(p++) != '=')
+            continue;
+        while (*p == ' ')
+            p += 1;
+        char delimiter = *p++;
+        if (delimiter != '\'' && delimiter != '"')
+            continue;
+
+        linkStart = p;
+        // [TODO] Handle escape char
+        while(*p != delimiter)
+            p++;
+        links.push_back({attr,std::string(linkStart, p)});
+        p += 1;
+    }
+    return links;
+}
+
+bool isOutofBounds(const std::string& input, std::string base)
+{
+    if (input.empty()) return false;
+
+    if (base.back() != '/')
+        base.push_back('/');
+
+    int nr = 0;
+    if (base.front() != '/')
+        nr++;
+
+    //count nr of substrings ../
+    int nrsteps = 0;
+    std::string::size_type pos = 0;
+    while((pos = input.find("../", pos)) != std::string::npos) {
+        nrsteps++;
+        pos += 3;
+    }
+
+    return nrsteps >= (nr + std::count(base.cbegin(), base.cend(), '/'));
+}
+
+int adler32(const std::string& buf)
+{
+    unsigned int s1 = 1;
+    unsigned int s2 = 0;
+    unsigned int sz=buf.size();
+    for (size_t n = 0; n <sz; n++)
+    {
+        s1 = (s1 + buf[n]) % 65521;
+        s2 = (s2 + s1) % 65521;
+    }
+    return (s2 << 16) | s1;
+}
+
+std::string normalize_link(const std::string& input, const std::string& baseUrl)
+{
+    std::string output;
+    output.reserve(baseUrl.size() + input.size() + 1);
+
+    bool in_query = false;
+    bool check_rel = false;
+    const char* p = input.c_str();
+    if ( *(p) == '/') {
+      // This is an absolute url.
+      p++;
+    } else {
+      //This is a relative url, use base url
+      output = baseUrl;
+      if (output.back() != '/')
+          output += '/';
+      check_rel = true;
+    }
+
+    //URL Decoding.
+    while (*p)
+    {
+        if ( !in_query && check_rel ) {
+            if (strncmp(p, "../", 3) == 0) {
+                // We must go "up"
+                // Remove the '/' at the end of output.
+                output.resize(output.size()-1);
+                // Remove the last part.
+                auto pos = output.find_last_of('/');
+                output.resize(pos==output.npos?0:pos);
+                // Move after the "..".
+                p += 2;
+                check_rel = false;
+                continue;
+            }
+            if (strncmp(p, "./", 2) == 0) {
+                // We must simply skip this part
+                // Simply move after the ".".
+                p += 2;
+                check_rel = false;
+                continue;
+            }
+        }
+        if ( *p == '#' || *p == '?')
+            // This is a beginning of the #anchor inside a page. No need to decode more
+            break;
+        if ( *p == '%')
+        {
+            char ch;
+            sscanf(p+1, "%2hhx", &ch);
+            output += ch;
+            p += 3;
+            continue;
+        }
+        if ( *p == '?' ) {
+            // We are in the query, so don't try to interprete '/' as path separator
+            in_query = true;
+        }
+        if ( *p == '/') {
+            check_rel = true;
+            if (output.empty()) {
+                // Do not add '/' at beginning of output
+                p++;
+                continue;
+            }
+        }
+        output += *(p++);
+    }
+    return output;
+}
