@@ -23,19 +23,53 @@
 
 #include <map>
 #include <string>
+#include <vector>
+#include <stdexcept>
+#include <sstream>
 
-std::string getMimeTypeForFile(const std::string& filename);
-std::string getNamespaceForMimeType(const std::string& mimeType);
+/* Formatter for std::exception what() message:
+ * throw std::runtime_error(
+ *   Formatter() << "zimwriterfs: Unable to read" << filename << ": " << strerror(errno));
+ */
+class Formatter
+{
+public:
+    Formatter() {}
+    ~Formatter() {}
+
+    template <typename Type>
+    Formatter & operator << (const Type & value)
+    {
+        stream_ << value;
+        return *this;
+    }
+
+    // std::string str() const         { return stream_.str(); }
+    operator std::string () const   { return stream_.str(); }
+
+private:
+    Formatter(const Formatter &);
+    Formatter & operator = (Formatter &);
+
+    std::stringstream stream_;
+};
+
+
+typedef struct html_link
+{
+    std::string attribute;
+    std::string link;
+} html_link;
+
+std::string getMimeTypeForFile(const std::string& basedir, const std::string& filename);
+std::string getNamespaceForMimeType(const std::string& mimeType, bool uniqueNamespace);
 std::string getFileContent(const std::string& path);
 unsigned int getFileSize(const std::string& path);
 std::string decodeUrl(const std::string& encodedUrl);
 std::string computeAbsolutePath(const std::string& path,
                                 const std::string& relativePath);
 bool fileExists(const std::string& path);
-std::string removeLastPathElement(const std::string& path,
-                                  const bool removePreSeparator,
-                                  const bool removePostSeparator);
-std::string computeNewUrl(const std::string& aid, const std::string& baseUrl, const std::string& targetUrl);
+bool isDirectory(const std::string &path);
 
 std::string base64_encode(unsigned char const* bytes_to_encode,
                           unsigned int in_len);
@@ -51,8 +85,21 @@ void stripTitleInvalidChars(std::string& str);
 std::string computeRelativePath(const std::string path,
                                 const std::string absolutePath);
 
-std::string removeAccents(const std::string& text);
-
 void remove_all(const std::string& path);
 
-#endif  // OPENZIM_TOOLS_H
+//Returns a vector of the links in a particular page. includes links under 'href' and 'src'
+std::vector<html_link> generic_getLinks(const std::string& page);
+
+// checks if a relative path is out of bounds (relative to base)
+bool isOutofBounds(const std::string& input, std::string base);
+
+//Adler32 Hash Function. Used to hash the BLOB data obtained from each article, for redundancy checks.
+//Please note that the adler32 hash function has a high number of collisions, and that the hash match is not taken as final.
+int adler32(const std::string& buf);
+
+//Removes extra spaces from URLs. Usually done by the browser, so web authors sometimes tend to ignore it.
+//Converts the %20 to space.Essential for comparing URLs.
+std::string normalize_link(const std::string& input, const std::string& baseUrl);
+
+
+#endif  // OPENZIM_TOOLS_H
